/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.backup;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.xml.object.XMLCollection;
import org.exoplatform.xml.object.XMLObject;
/**
 * May 27, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $ID$
 **/
abstract public class ImportExportPlugin implements  ComponentPlugin {
  
  final static public String DATA_VERSION = "1.0.1" ;
  
  private Map<String, XMLObjectConverter> 
            xmlObjectTransformer_= new HashMap<String, XMLObjectConverter>(); ;
  private List<DataTransformer> dataTransformer_ = new ArrayList<DataTransformer>() ;
  
  private String name ;
  private String description ;
  private String dataVersion ;
  
  public void init(InitParams params) throws Exception {
    List<DataTransformer> list = params.getObjectParamValues( DataTransformer.class) ;
    addDataTransformer( list) ;
    List<XMLObjectConverter> list2 = params.getObjectParamValues(XMLObjectConverter.class) ;
    addXMLObjectConverter( list2) ;
    if(params.getValueParam("data.version") != null) {
      dataVersion = params.getValueParam("data.version").getValue() ;
    } 
  }
  
  public String getName() { return name ; }
  public void   setName(String s) { name = s ; }
  
  public String getDescription() { return description ; }
  public void   setDescription(String s) { description =  s ; }
  
  public String getCurrentDataVersion() { return dataVersion ; }
  public void   setCurrentDataVersion(String s) { dataVersion = s ; }
  
  public boolean hasUserData()  { return true ;  }
  public boolean hasServiceData() { return true ; }
  
  public void addDataTransformer(List<DataTransformer> list) throws Exception {
    for(int i = 0; i < list.size() ; i++) {
      DataTransformer transformer = list.get(i) ;
      transformer.init(this) ;
      dataTransformer_.add(transformer) ;
    }
  }
  
  public List<DataTransformer> getDataTransformer() { return dataTransformer_ ; }
  
  public void addXMLObjectConverter(List<XMLObjectConverter> list) {
    for(int i = 0; i < list.size() ; i++) {
      XMLObjectConverter transformer = list.get(i) ;
      xmlObjectTransformer_.put(transformer.getDataVersion(), transformer) ;
    }
  }
  
  public XMLObjectConverter getXMLObjectConverter(String dataVersion) {
    return xmlObjectTransformer_.get( dataVersion) ;
  }
  
  abstract public void exportUserData(String user, ZipOutputStream os) throws Exception ;
  abstract public void importUserData(String user, ZipFile in) throws Exception ;
  
  public void transformUserData(String user, ZipOutputStream os) throws Exception {
    for(int i = 0; i < dataTransformer_.size(); i++) {
      DataTransformer transformer = dataTransformer_.get(i) ;
      transformer.transformUserData(user, os) ;
    }
  }
  
  abstract public void exportServiceData( ZipOutputStream os) throws Exception ;
  abstract public void importServiceData( ZipFile in) throws Exception ;
  
  public void transformServiceData( ZipOutputStream os) throws Exception {
    for(int i = 0; i < dataTransformer_.size(); i++) {
      DataTransformer transformer =  dataTransformer_.get(i) ;     
      transformer.transformServiceData( os); 
    }
  }
  
  public Metadata getMetadata(String name, ZipFile zipFile) throws Exception {
    Metadata mdata = (Metadata) XMLObject.getObject(getEntry(name, zipFile)) ;
    return mdata ;
  }
  
  protected InputStream getEntry(String name, ZipFile zipFile) throws Exception {
    ZipEntry entry = zipFile.getEntry(name);
    if( entry == null) return null;
    return zipFile.getInputStream(entry) ;
  }
  
  protected void createEntry(String name, ZipOutputStream os, Object o) throws Exception  {
    os.putNextEntry(new ZipEntry(name));
    if(o instanceof Collection) {
      XMLCollection xmlobject = new XMLCollection((Collection)o) ;
      os.write(xmlobject.toByteArray("UTF-8")) ;
    } else {
      XMLObject xmlobject = new XMLObject(o) ;
      os.write(xmlobject.toByteArray("UTF-8")) ;
    }
    os.closeEntry() ;
  }
}