/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.naming;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameAlreadyBoundException;
import javax.naming.NamingException;
import javax.naming.Reference;

import org.apache.commons.logging.Log;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.configuration.ConfigurationException;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: InitialContextInitializer.java 6677 2006-07-03 10:49:59Z geaz $
 */

public class InitialContextInitializer {
  
  private static Log log = ExoLogger.getLogger("naming.InitialContextInitializer");

  private List <BindReferencePlugin> bindReferencesPlugins;
  
  private String defaultContextFactory;
  
  private Context initialContext;
  
  public InitialContextInitializer(InitParams params) throws NamingException, 
  ConfigurationException {
    

    if(System.getProperty(Context.INITIAL_CONTEXT_FACTORY) == null) {
      ValueParam defFactoryParam = params.getValueParam("default-context-factory");
      if(defFactoryParam == null)
        throw new ConfigurationException("No 'default-context-factory' parameter");
      defaultContextFactory = defFactoryParam.getValue();
      System.setProperty(Context.INITIAL_CONTEXT_FACTORY , defaultContextFactory);
    }
    log.info("InitialContextFactory: "+System.getProperty(Context.INITIAL_CONTEXT_FACTORY));

    initialContext = new InitialContext();

    bindReferencesPlugins = new ArrayList<BindReferencePlugin>();

  }

  
  // for out-of-container testing
  private InitialContextInitializer(String name, Reference reference) throws NamingException {
    if(System.getProperty(Context.INITIAL_CONTEXT_FACTORY) == null)
      System.setProperty(Context.INITIAL_CONTEXT_FACTORY,
          defaultContextFactory);
    initialContext = new InitialContext();
    initialContext.rebind(name, reference);
    
  }
  
  /**
   * Patch for case when bound objects are not shared i.e. there are some parts of app using 
   * different copy of Context, for example per web app InitialContext in Tomcat  
   */
  public void recall() {
    for(BindReferencePlugin plugin : bindReferencesPlugins) {
      try {
        InitialContext ic = new InitialContext();
        ic.bind(plugin.getBindName(), plugin.getReference());
        log.info("Reference bound (by recall()): "+plugin.getBindName());
      } catch (NameAlreadyBoundException e) {
        log.debug("Name already bound: "+plugin.getBindName());
      } catch (NamingException e) {
        log.error("Could not bind: "+plugin.getBindName(), e);
        //e.printStackTrace();
      }
    }
  }
  
  public void addPlugin(ComponentPlugin plugin) {
    if (plugin instanceof BindReferencePlugin) {
      BindReferencePlugin brplugin = (BindReferencePlugin) plugin;
      try {
        //initialContext = new InitialContext();
        initialContext.rebind(brplugin.getBindName(), brplugin.getReference());
        log.info("Reference bound: "+brplugin.getBindName());
        bindReferencesPlugins.add((BindReferencePlugin)plugin);
      } catch (NamingException e) {
        log.error("Could not bind: "+brplugin.getBindName(), e);
        //e.printStackTrace();
      }
    }
  }

  public ComponentPlugin removePlugin(String name) {
    return null;
  }

  public Collection getPlugins() {
    return bindReferencesPlugins;
  }
  
  public String getDefaultContextFactory() {
    return defaultContextFactory;
  }


  public Context getInitialContext() {
    return initialContext;
  }

  // for out-of-container testing
  public static void initialize(String name, Reference reference) throws NamingException {
    new InitialContextInitializer(name, reference);
  }

  
}
