/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.transaction.impl.jotm;

import java.rmi.RemoteException;

import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.UserTransaction;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.naming.InitialContextInitializer;
import org.exoplatform.services.transaction.TransactionService;
import org.objectweb.jotm.Current;
import org.objectweb.jotm.TransactionFactory;
import org.objectweb.jotm.TransactionFactoryImpl;
import org.objectweb.jotm.XidImpl;
import org.objectweb.transaction.jta.ResourceManagerEvent;


/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */
 
public class TransactionServiceJotmImpl implements TransactionService {
  
  private Current current;
  
  public TransactionServiceJotmImpl(InitialContextInitializer initializer, 
      InitParams params) throws RemoteException {
    TransactionFactory tm = new TransactionFactoryImpl();
    current = new Current(tm);
    if(params != null && params.getValueParam("timeout") != null) {
      int t = Integer.parseInt(params.getValueParam("timeout").getValue()); 
      current.setDefaultTimeout(t);
    }
  }

  public TransactionManager getTransactionManager() {
    return current;
  }

  public UserTransaction getUserTransaction() {
    return current;
  }
  
  public void enlistResource(XAResource xares) throws RollbackException, SystemException {
    Transaction tx = getTransactionManager().getTransaction();
    if(tx != null)
      current.getTransaction().enlistResource(xares);
    else
      current.connectionOpened((ResourceManagerEvent)xares);
  }

  public void delistResource(XAResource xares) throws RollbackException, SystemException {
    Transaction tx = getTransactionManager().getTransaction();
    if(tx != null)
      current.getTransaction().delistResource(xares, XAResource.TMNOFLAGS);
    else
      current.connectionClosed((ResourceManagerEvent)xares);
  }

  public Xid createXid() {
    return new XidImpl();
  }
  
  public int getDefaultTimeout() {
    return current.getDefaultTimeout();
  }

  public void setTransactionTimeout(int seconds) throws SystemException {
    current.setTransactionTimeout(seconds);
  }

}
