/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.database.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.naming.InitialContextInitializer;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: DBSchemaCreator.java 6688 2006-07-03 16:08:06Z peterit $
 */

public class DBSchemaCreator  {
  
  static public String SQL_DELIMITER_COMMENT_PREFIX = "/*$DELIMITER:"; 
  static public String SQL_DELIMITER = ";";
  
  private static Log log = ExoLogger.getLogger("database.DBSchemaCreator");
  
  private List <CreateDBSchemaPlugin> createDBSchemaPlugins = new ArrayList<CreateDBSchemaPlugin>();
  
  public DBSchemaCreator(InitialContextInitializer contextInit) {

  }
  
  // for testing only
  private DBSchemaCreator(String dsName, String script) throws SQLException, NamingException {
    createTables(dsName, script);
  }

  public void createTables(String dsName, String script) throws NamingException, SQLException {
    InitialContext context = new InitialContext();
    Object obj = context.lookup(dsName);
    
    DataSource ds;
    
    ds = (DataSource)context.lookup(dsName);
    
    Connection conn = ds.getConnection();
    conn.setAutoCommit(false);
    String sql = "";
    try {
      String[] scripts = null;
      if (script.startsWith(SQL_DELIMITER_COMMENT_PREFIX)) {
        // read custom prefix
        try {
          String s = script.substring(SQL_DELIMITER_COMMENT_PREFIX.length());
          int endOfDelimIndex = s.indexOf("*/");
          String delim = s.substring(0, endOfDelimIndex).trim();
          s = s.substring(endOfDelimIndex+2).trim();
          scripts = s.split(delim);
        } catch(IndexOutOfBoundsException e) {
          log.warn("Error of parse SQL-script file. Invalid DELIMITER configuration. Valid format is '"
              + SQL_DELIMITER_COMMENT_PREFIX + "XXX*/' at begin of the SQL-script file, where XXX - DELIMITER string."
              + " Spaces will be trimed. ", e);
          log.info("Using DELIMITER:[" + SQL_DELIMITER + "]");
          scripts = script.split(SQL_DELIMITER);
        }
      } else {
        scripts = script.split(SQL_DELIMITER);
      }
      for (String scr: scripts) {
        String s = cleanWhitespaces(scr.trim());
        if (s.length()>0) {
          sql = s;
          log.debug("Execute script: \n[" + sql+"]");
          conn.createStatement().executeUpdate(sql);
        }
      }
      conn.commit();
      log.info("DB schema of DataSource: '"+dsName+"' created succesfully. context "+context);
    } catch (SQLException e) {
      conn.rollback();  
      SQLException next = e.getNextException(); 
      String errorTrace = "";
      while (next != null) {
        errorTrace += next.getMessage() + "; ";
        next = e.getNextException();
      }
      Throwable cause = e.getCause(); 
      log.error("Could not create db schema of DataSource: '" + dsName 
          + "'. Reason: " + e.getMessage() + "; " + errorTrace
          + (cause != null ? " (Cause: " + cause.getMessage() + ")" : "") 
          + ". Last command: " + sql);
    } finally {
      conn.close();
    }
  }
  
  public void addPlugin(ComponentPlugin plugin) {
    if (plugin instanceof CreateDBSchemaPlugin) {
      CreateDBSchemaPlugin csplugin = (CreateDBSchemaPlugin) plugin;
      try {
        createTables(csplugin.getDataSource(), csplugin.getScript());
        createDBSchemaPlugins.add(csplugin);
      } catch (NamingException e) {
        e.printStackTrace();
      } catch (SQLException e) {
        e.printStackTrace();
      }
    }      
  }

  public ComponentPlugin removePlugin(String name) {
    return null;
  }

  public Collection getPlugins() {
    return createDBSchemaPlugins;
  }
  
  // for testing
  public static DBSchemaCreator initialize(String dsName, String script) throws SQLException, NamingException {
    return new DBSchemaCreator(dsName, script);
  }
  
  static public String cleanWhitespaces(String string) {
    if (string != null) {
      char[] cc = string.toCharArray();
      for (int ci = cc.length - 1; ci > 0; ci--) {
        if (Character.isWhitespace(cc[ci])) {
          cc[ci] = ' ';
        }
      }
      return new String(cc);
    }
    return string; 
  }  

}
