/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.ticket;


import java.text.SimpleDateFormat;
import java.util.Date;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.ticket.impl.TicketImpl;
import org.exoplatform.test.BasicTestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 5, 2006
 */
public class TestTicketService extends BasicTestCase {
  protected static TicketService service_;

  public TestTicketService(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    if (service_ == null) {
      service_ = (TicketService)PortalContainer.getInstance().getComponentInstanceOfType(TicketService.class);
    }
  }
  
  public void testTicketService() throws Exception {
    assertTrue("Found the ticket service in  the portal container", service_ !=  null) ;
    SimpleDateFormat fomatDate=new SimpleDateFormat();
    TicketCommand tc = new TestTicketCommand() ;
    tc.setProperty("param1","platform");
    tc.setProperty("param2","chungnv");
    String[] allowedUsers = {"exo" , "dung","chungnv","tester","owen"} ;
    String[] permissions = {"member:user", "member:/admin"} ;
    String owenCreator="nguyenchung136";
    String desc="Download";
    
    //-------------------Test Ticket with Users----------------------------//
    String id=service_.createTicket(110000,4,desc,owenCreator, allowedUsers, permissions,tc);
    Date dateEquals=new Date();
    Long lastTime=dateEquals.getTime()+110000;
    Date lastDate=new Date(lastTime);
    /**
     * Get Ticket with TicketID 
     * AccessCount = 0 because method execute() do not call 
     */
    Ticket ticket=service_.getTicket(id);
    TicketImpl ticketImpl=(TicketImpl)ticket;
    assertTrue("Expect one ticket is  found", ticket != null) ;
    assertEquals("Expect the a correct live time", ticket.getLiveTime(), 110000) ;
    assertEquals("Expect the a correct repeat count",4, ticket.getLimitAccess()) ;
    assertEquals("Expect the a correct AcessCount",0,ticket.getAccessCounter());
    assertEquals("Expect the a correct Desc",desc,ticketImpl.getDescription());
    assertEquals("Expect the a correct Creator","nguyenchung136",ticket.getCreator());
    assertEquals("Expect the a correct Property of TicketCommand","platform",tc.execute());
    assertEquals("Expect the a correct Property of Ticket",tc.execute(),ticketImpl.getProperties());
    assertEquals("Expect the a correct Number User",5 , ticket.getAlowUsers().length) ;
    assertEquals("Expect the a correct Number Permisson",2 , ticket.getRequiredPermissions().length) ;
    assertEquals("Expect the a correct Status",Ticket.VALID_STATUS , ticket.getStatus()) ;
    assertEquals("Expect the a correct time creantion",fomatDate.format(dateEquals) ,
                                                       fomatDate.format(ticket.getCreationTime())) ;
    assertEquals("Expect the a correct Last Time",fomatDate.format(lastDate) ,
                                                  fomatDate.format(ticket.getLastAccessTime()));
    
    /**
     * Get Ticket with TicketID and User
     * Test methods of the ticket 
     */
     Ticket ticketIU=service_.getTicket(id,allowedUsers[4]);
     assertTrue("Expect one ticketIU is  found with ticketIU", ticketIU !=null) ;
     assertEquals("Expect the a correct live time", ticketIU.getLiveTime(), 110000) ;
     assertEquals("Expect the a correct repeat count with ticketIU", 4, ticketIU.getLimitAccess()) ;
     assertEquals("Expect the a correct Access Count ticketIU", 0,ticketIU.getAccessCounter());
     assertEquals("nguyenchung136",ticketIU.getCreator());
     assertEquals("Expect the a correct length User with ticketIU",5 , ticketIU.getAlowUsers().length) ;
     assertEquals("Expect the a correct length Permissions with ticketIU",2 ,ticketIU.getRequiredPermissions().length) ;
     assertEquals("Expect the a correct Status Lock of Service with ticketIU ",Ticket.VALID_STATUS , ticketIU.getStatus()) ;
     assertEquals("Expect the a correct time creantion", fomatDate.format(dateEquals) , 
                                                         fomatDate.format(ticketIU.getCreationTime())) ;
     assertEquals("Expect the a correct Last Time",fomatDate.format(lastDate) ,
                                                   fomatDate.format(ticketIU.getLastAccessTime()));
     
     
     // Before call method execute(). AccessCount don't Increase and User login not Save in DataBase 
     
     assertEquals("Expect the a correct User Access",ticketIU.getLastAccessUser(),null);
     // After call method excude(). AccessCount Increase and Access User Save in DataBase
     service_.executeTicketCommand(id,allowedUsers[4]);
     assertEquals("Expect the a correct Access Count with ticketIU", 1,ticketIU.getAccessCounter());
     assertEquals("Expect the a correct User Access with ticketIU",ticketIU.getLastAccessUser(),allowedUsers[4]);
     
     /**
      * Test Users 
      * Get Ticket with array Users
      */
     for(int i=0;i<allowedUsers.length;i++){
       Ticket ticketUsers=service_.getTicket(id,allowedUsers[i]);
       assertTrue("Expect one ticketIU is  found", ticketUsers !=null) ;
     }
     /**
      * Test Methods in TicketService
      * AccessCount will Increase 1 because method execute called 
      * AccessCount = 2 
      */
     service_.executeTicketCommand(id,allowedUsers[1]);
     assertEquals(2,ticketIU.getAccessCounter());
     /**
      * AccessCount will Increase 1 because method execute called
      * AccessCount = 3 
      */
     service_.executeTicketCommand(ticket,allowedUsers[0]);
     /**
      * AccessCount will Increase 1 because method execute called
      * AccessCount = 4
      * After run Status =2 , ticket Locked
      */
     Ticket ticketLock=service_.getTicket(id,allowedUsers[3]);
     assertTrue("Expect one ticketLock is  Locked", ticketLock!=null) ;
     service_.executeTicketCommand(ticketLock,allowedUsers[3]);
      assertEquals(4,ticketLock.getAccessCounter());
      assertEquals("Expect LOCKED_STATUS ticketLocker", Ticket.LOCKED_STATUS , ticketLock.getStatus()) ;
      /**
       * Locker Ticket
       * Status = 1 
       */
      service_.executeTicketCommand(ticketLock,allowedUsers[1]);
      assertEquals("Expect ticketLocker", Ticket.LOCKED_STATUS, ticketLock.getStatus()) ;
      
      service_.executeTicketCommand(ticketLock,allowedUsers[1]);
      assertEquals(4,ticketLock.getAccessCounter());
      
      /**
       * Ticket Locked
       * 
       */
      Ticket ticketLocked=service_.getTicket(id,allowedUsers[2]);
      assertTrue("Expect one ticketLocked is  Locked", ticketLocked != null) ;
      assertEquals("Expect the a correct Status ticketIU",ticketIU.getStatus(),Ticket.LOCKED_STATUS);
      
      // The lass UserAccess is allowedUsers[3] 
      assertEquals("Expect the a correct User Access",ticketIU.getLastAccessUser(),allowedUsers[3]);
      
      
      //--------Test   Ticket with AlowUser=null----Creator=null------Permission=null-----------//
      String ticketIdNull=service_.createTicket(10000,5,desc,tc);
      Date createDate=new Date();
      Ticket ticketNull=service_.getTicket(ticketIdNull);
      assertTrue("Expect one ticket is  found", ticketNull != null) ;
      assertEquals("Expect the a correct live time", ticketNull.getLiveTime(), 10000);
      assertEquals("Expect the a correct limit Access", ticketNull.getLimitAccess(),5);
      assertEquals("Expect the a correct Creator",ticketNull.getCreator(),null);
      assertEquals("Expect the a correct Time Create",fomatDate.format(ticketNull.getCreationTime()),
                                                      fomatDate.format(createDate));
      assertEquals("Expect the a correct AlowUsers",ticketNull.getAlowUsers(),null);
      assertEquals("Expect the a correct Permission",ticketNull.getRequiredPermissions(),null);
      assertEquals("Expect the a correct Status Lock",ticketNull.getStatus(),Ticket.VALID_STATUS);
      assertEquals("Expect the a correct AccessCount ticketNull",ticketNull.getAccessCounter(),0);
      
      service_.executeTicketCommand(ticketIdNull);
      assertEquals("Expect the a correct Access User",ticketNull.getLastAccessUser(),null);
      assertEquals("Expect the a correct Access Count",ticketNull.getAccessCounter(),1);
      
      //---------Test Ticket with Creator=null-----AlowUsers=null------------------------------//
      String ticketIdCA=service_.createTicket(10100,5,desc,null,null,permissions,tc);
      Date createDateCA=new Date();
      Ticket ticketNullCA=service_.getTicket(ticketIdCA);
      assertTrue("Expect one ticket is  found", ticketNullCA != null) ;
      assertEquals("Expect the a correct live time", ticketNullCA.getLiveTime(), 10100);
      assertEquals("Expect the a correct limit Access", ticketNullCA.getLimitAccess(),5);
      assertEquals("Expect the a correct AccessCount ticketNullCA", 0,ticketNullCA.getAccessCounter());
      assertEquals("Expect the a correct Creator",ticketNull.getCreator(),null);
      assertEquals("Expect the a correct Time Create",fomatDate.format(ticketNullCA.getCreationTime()),
                                                      fomatDate.format(createDateCA));
      assertEquals("Expect the a correct AlowUsers",ticketNullCA.getAlowUsers(),null);
      assertEquals("Expect the a correct Permission",ticketNullCA.getRequiredPermissions().length,2);
      assertEquals("Expect the a correct Status Lock",ticketNullCA.getStatus(),Ticket.VALID_STATUS);
      
      service_.executeTicketCommand(ticketNullCA);
      assertEquals("Expect the a correct Access User",ticketNullCA.getLastAccessUser(),null);
      assertEquals("Expect the a correct Access Count",ticketNullCA.getAccessCounter(),1);
      
      //--------------Test Lock Ticket with Time  -----------------------------------//
      // decrease liveTime of Service so lower so Now Time,bring service to Locked
      
      TicketImpl impl=(TicketImpl)ticketNullCA;
      Long timeSetCreation=createDateCA.getTime()-115000;
      impl.setCreationTime(new Date(timeSetCreation));
      // Execute() , bring service to Locked 
      Object obLock=service_.executeTicketCommand(impl);
      assertEquals("Expect the a correct ticketNullCA Locked with Status", Ticket.EXPIRED_STATUS , ticketNullCA.getStatus()) ;
      assertEquals("Expect the a correct Object execute()",obLock ,null) ;
      
      // Test Lock Ticket with Time (Thear )
      String idTime=service_.createTicket(2000,10,desc,owenCreator,null,permissions,tc);
      Ticket ticketTime=service_.getTicket(idTime);
      assertTrue("Expect one ticket is  found", ticketTime != null) ;
      assertEquals("Expect the a correct live time", ticketTime.getLiveTime(),2000);
      service_.executeTicketCommand(ticketTime);
      assertEquals("Expect the a correct Access Count", ticketTime.getAccessCounter(),1);
     
      
      /** After finis Long time !
       *  Status = 1, call execute() return NULL
       */
      Thread.sleep(2100);
      Object ob= service_.executeTicketCommand(ticketTime);
      assertTrue("Expect one ticketTime is  found", ticketTime != null) ;
      assertEquals("Expect the a correct Status ticketTime", ticketTime.getStatus(),Ticket.EXPIRED_STATUS);
      assertEquals("Expect the a correct Access Count",ob,null);
      assertEquals("Expect the a correct Access Count", ticketTime.getAccessCounter(),1);
      //--------------------Test LimitAccessCount----------------------------------------------
      String idLimit=service_.createTicket(110000,Ticket.UNLIMITED_ACCESS,desc,null, null,null,tc);
      Ticket ticketLimit=service_.getTicket(idLimit);
      assertTrue("Expect one ticketLimit is  found", ticketLimit != null) ;
      assertEquals("Expect the a correct Status ticketLimit", ticketLimit.getStatus(),Ticket.VALID_STATUS);
      // Access Limit = -1 ----> UNLIMITED 
      assertEquals("Expect the a correct Access Limit", ticketLimit.getLimitAccess(),Ticket.UNLIMITED_ACCESS);
      for(int i=0;i<6;i++){ service_.executeTicketCommand(idLimit);  }
      assertEquals("Expect the a correct Access Count", ticketLimit.getAccessCounter(),6);
      Object obTest=service_.executeTicketCommand(idLimit);
      assertEquals("Expect the a correct Last User Accep", ticketLimit.getLastAccessUser(),null);
      //--------------- Ben Ticket ServiceImp phan execute return null
      assertEquals("Expect the a correct Object Execute Limit", obTest,null);
      
  }
  
  static public class TestTicketCommand extends  TicketCommand {
    public Object execute() throws Exception {
      return  getProperty("param1");
    }

   
    public String getType() throws Exception {
      return null;
    }
  }
}