/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.templates.groovy;

import java.io.Writer;
import java.util.Map;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 2, 2005
 * @version $Id: GroovyTemplateService.java 5799 2006-05-28 17:55:42Z geaz $
 * 
 * This service is used to manage the groovy template. The groovy template can be stored 
 * in many different media such the file system , a jar  file or database.....
 */
public interface GroovyTemplateService {
  /**
   * Register a template loader. A template loader can load one or many template 
   * @param loader
   */
  public void addTemplateLoader(GroovyTemplateLoader loader)  ;
  /**
   * Search and load the template according to the template type, name and produce 
   * the output text 
   * @param type The type of the template, for example the template should be in file system, 
   * classpath or database.. The service should search for a template that match the name but
   * type may help it  find the template faster.
   * @param name The name of the template , You can have many template with the same name. But
   * the service will return the first template it  can find.
   * @param binding The input objects that  should be merged with the template.
   * @return  The output text
   * @throws TemplateNotFoundException
   * @throws Exception
   */
  public String merge(String type, String name, Map  binding) 
  throws TemplateNotFoundException, Exception ;
  /**
   * This method should do the same thing with public String merge(String type, String name, Map  binding)
   * but write the output text to the writer instead return the output text.
   * @param writer
   * @param type
   * @param tname
   * @param binding
   * @throws TemplateNotFoundException
   * @throws Exception
   */
  public void merge(Writer writer, String type, String tname, Map  binding) 
  throws TemplateNotFoundException, Exception ;
  /**
   * Use this method  to merge  the output of one template with an other template.
   * For example:
   * 
   * out put of template1 <% service.include("file", "template2", binding) %> end
   * 
   * @param type
   * @param name
   * @param binding
   * @return
   * @throws TemplateNotFoundException
   * @throws Exception
   */
  public String include(String type, String name, Map  binding) 
  throws TemplateNotFoundException,Exception ;
  
  /**
   * Use this method  to merge  the output of one template with an other template.
   * For example:
   * 
   * out put of template1 <% service.include("template2", binding) %> end
   * 
   * @param name
   * @param binding
   * @return
   * @throws TemplateNotFoundException
   * @throws Exception
   */
  public String include(String name, Map  binding) 
  throws TemplateNotFoundException,Exception ;
}