/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.List;
import java.util.ResourceBundle;

import org.exoplatform.commons.utils.ExpressionUtil;
import org.exoplatform.portal.PortalConstants;
import org.exoplatform.portal.config.model.Component;
import org.exoplatform.portal.config.model.Container;
import org.exoplatform.portal.config.model.Page;
import org.exoplatform.portal.faces.listener.page.PageActionListener;
import org.exoplatform.portal.skin.SkinConfigService;
import org.exoplatform.portal.skin.model.Style;

/**
 * Fri, May 30, 2003 @
 * @author : Mestrallet Benjamin
 * @email:   benjmestrallet@users.sourceforge.net
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPage.java,v 1.6 2004/09/26 19:14:42 tuan08 Exp $
 */
public class UIPage extends UIContainer {
  private Page componentModel_ ;
  private UIPortlet maximizedPortlet_ ; 
  private boolean dirty_ ;
  private String error_ = null ;

  public UIPage() {
    setRendererType(getJavaRendererType()) ;
  }
  
  public UIPage(Page config, String defaultStyle) {
    componentModel_ = config ;
    if(config.getId() == null) config.setId("page");
    setBasicProperties(config, defaultStyle) ;
    List childrenConfig = config.getChildren() ;
    initChildren(childrenConfig, config.getPageId()) ;
    registerListeners();
  }
  
  private void registerListeners() {
    addActionListener(PageActionListener.class, PortalConstants.DRAG_DROP_ACTION) ;
  }
  
  public String getIdPrefix() { return "page" ; }
  protected String getSkinName() { return "default" ; } 
  
  /**Override default setDirty in UIBasicComponent*/
  public void setComponentDirty(boolean b) { dirty_ = b ; }
  public boolean isDirty() { return dirty_ ; }
  
  public Component getComponentModel() { return componentModel_ ; }
  public Page      getPageModel() { return componentModel_ ; }
  public Page getEditablePageModel() {
  	if(!modified_) {
  		componentModel_ = (Page)componentModel_.softCloneObject() ;
  	}
  	return componentModel_ ;
  }
  
  public String getViewPermission() { return null ;}
  public String getEditPermission() { return null ;}
  public String getOwner() { return componentModel_.getOwner() ;}
  
  public boolean hasError() { return error_ != null ; }
  public String getError() { return error_ ;}
  public void   setError(String error) { error_ =  error ; }
  
  public UIPortlet getMaximizedPortlet() { return maximizedPortlet_ ; }
  public void      setMaximizedPortlet(UIPortlet uiPortlet) { maximizedPortlet_ = uiPortlet ; }

  public String getDefaultRendererName() { return "PageRowRenderer" ; }
  public String getJavaRendererType() { return  "PageRenderer" ; }

  public Style getDecoratorStyle(SkinConfigService service, String renderer, String style) {
    return service.getPageDecoratorStyle(renderer, style);
  }
  
  public void buildTreeModel(Container parent) {
  	componentModel_.getChildren().clear() ;
  	List uichildren = getChildren() ;
  	for(int i = 0; i < uichildren.size(); i++) {
  		UIPortalComponent uiChild =(UIPortalComponent) uichildren.get(i) ;
  		uiChild.buildTreeModel(componentModel_) ;
  	}
  	if(parent != null) parent.getChildren().add(componentModel_) ;
  }
  
  public void changeLocale(ResourceBundle res) {
    String title = componentModel_.getTitle() ;
    if(title == null) displayTitle_ = getId() ;
    else  displayTitle_ = ExpressionUtil.getExpressionValue(res, title ) ;
  }
  
  public String getTemplatePath() {
    return "org/exoplatform/portal/faces/renderer/html/page/UIPage.vm" ;
  }
}