/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.log;

import java.util.Date;


/**
 * Created by The eXo Platform SARL
 * Author : Hung Nguyen
 *          nqhungvn@users.sourceforge.net
 * Sep 26, 2005
 * @hibernate.class  table="PORTLET_LOG_DATA"
 */
public class PortletLogData {
  private String id ;
  private Date timeSlot ;
  private int timeIndex ;
  private String strTimeSlot ;
  private String strTimeIndex ;
  private String portletId ;
  private String portletAppId ;
  
  private long renderCounter ;
  private long renderErrorCounter ;
  private long sumRenderTime ;
  
  private long processActionCounter ;
  private long processActionErrorCounter ;
  private long sumProcessActionTime ;
  
  private boolean dirty ;
  
  public PortletLogData(){
    renderCounter = 0 ;
    renderErrorCounter = 0 ;
    sumRenderTime = 0 ;
    processActionCounter = 0 ;
    processActionErrorCounter = 0 ;
    sumProcessActionTime = 0 ;
  }
  
  /**
   * @hibernate.id  generator-class="assigned" unsaved-value="null"
   ***/
  public String getId() { return id ; }
  public void   setId(String s) { id = s ; }
  
   /**
   * @hibernate.property
   ***/
  public Date   getTimeSlot() { return timeSlot ; }
  public void   setTimeSlot(Date d) { timeSlot = d ; }
  
  /**
   * @hibernate.property
   ***/
  public int   getTimeIndex() { return timeIndex ; }
  public void   setTimeIndex(int i) { timeIndex = i ; }
  
   /**
   * @hibernate.property
   ***/
  public String getPortletId() { return portletId ; }
  public void   setPortletId(String s) { portletId = s ; }
  
   /**
   * @hibernate.property
   ***/
  public String getStrTimeSlot() { return strTimeSlot ; }
  public void   setStrTimeSlot(String s) { strTimeSlot = s ; }
  
   /**
   * @hibernate.property
   ***/
  public String getStrTimeIndex() { return strTimeIndex ; }
  public void   setStrTimeIndex(String s) { strTimeIndex = s ; }
  
   /**
   * @hibernate.property
   ***/
  public String getPortletAppId() { return portletAppId ; }
  public void   setPortletAppId(String s) { portletAppId = s ; }
  
  /**
   * @hibernate.property
   ***/
  public long getRenderCounter() {  return renderCounter ; }
  public void setRenderCounter(long c) { renderCounter = c  ; }
  
  /**
   * @hibernate.property
   ***/
  public long getRenderErrorCounter() {  return renderErrorCounter  ; }
  public void setRenderErrorCounter(long c) {  renderErrorCounter = c  ; }
  
  /**
   * @hibernate.property
   ***/
  public long getSumRenderTime() {  return sumRenderTime  ; }
  public void setSumRenderTime(long c) {  sumRenderTime = c  ; }
  
  /**
   * @hibernate.property
   ***/
  public long getProcessActionCounter() {  return processActionCounter ; }
  public void setProcessActionCounter(long c) {  processActionCounter = c ; }
  
  /**
   * @hibernate.property
   ***/
  public long getProcessActionErrorCounter() {  return processActionErrorCounter  ; }
  public void setProcessActionErrorCounter(long c) {  processActionErrorCounter = c  ; }
  
  /**
   * @hibernate.property
   ***/
  public long getSumProcessActionTime() {  return sumProcessActionTime  ; }
  public void setSumProcessActionTime(long c) {  sumProcessActionTime = c  ; }
  
  
  public boolean isDirty() {  return dirty ; }
  public void    setDirty(boolean b) { dirty = b ; }
  
  public long getAvgRenderTime() {
    if(renderCounter == 0) return 0 ;
    return sumRenderTime/renderCounter ; 
  }
  
  public String getShortDateTime(){
    String dateTime =  String.valueOf(timeSlot.getDate())+ "-" + 
                       String.valueOf(timeSlot.getMonth() + 1) + "-" + 
                       String.valueOf(timeSlot.getYear()+ 1900) + " / " + 
                       String.valueOf(timeIndex) ;
    return dateTime ;
  }
  
  public long getAvgProcessActionTime() {
    if(processActionCounter == 0) return 0 ;
    return sumProcessActionTime/processActionCounter ;
  }
  
  synchronized public void logPortletRender(String uniqueId, String pId, String appId, Date timeSl, 
                                            int index, long exeTime, boolean error) {
    
    id           = uniqueId ; 
    portletId    = pId ;
    portletAppId = appId ;
    timeSlot     = timeSl ;
    timeIndex    = index ;
    strTimeSlot  = timeSl.toString() ;
    strTimeIndex = String.valueOf(index) ;
    renderCounter++ ;
    if(error) 
      renderErrorCounter++ ;
    sumRenderTime += exeTime ;
  }
  
  synchronized public void updatePortletRender(long exeTime, boolean error) {
    renderCounter++ ;
    if(error) 
      renderErrorCounter++ ;
    sumRenderTime += exeTime ;
  }
  
  synchronized public void logPortletAction(String uniqueId, String pId, String appId, Date timeSl, 
                                            int index, long exeTime, boolean error) {
    id           = uniqueId ; 
    portletId    = pId ;
    portletAppId = appId ;
    timeSlot     = timeSl ;
    timeIndex    = index ;
    strTimeSlot  = timeSl.toString() ;
    strTimeIndex = String.valueOf(index) ;
    processActionCounter++ ;
    if(error) processActionErrorCounter++ ;
    sumProcessActionTime += exeTime ;
  }
  
  synchronized public void updatePortletAction(long exeTime, boolean error) {
    processActionCounter++ ;
    if(error) 
      processActionErrorCounter++ ;
    sumProcessActionTime += exeTime ;
  }
  
  synchronized public void merge(PortletLogData other) {
    renderCounter += other.getRenderCounter() ;
    renderErrorCounter += other.getRenderErrorCounter() ;
    sumRenderTime += other.getSumRenderTime() ;
    
    processActionCounter += other.getProcessActionCounter() ;
    processActionErrorCounter += other.getProcessActionErrorCounter() ;
    sumProcessActionTime += other.getSumProcessActionTime() ;
  }
}