package org.exoplatform.portlets.user.component;

import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionListener;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Util;
import org.exoplatform.faces.core.component.* ;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.EmailAddressValidator;
import org.exoplatform.faces.core.validator.EmptyFieldValidator;
import org.exoplatform.faces.core.validator.IdentifierValidator;
import org.exoplatform.faces.core.validator.NameValidator;
import org.exoplatform.container.PortalContainer ;
import org.exoplatform.portal.Information;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIAccountForm.java,v 1.14 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIAccountForm extends UISimpleForm {
  private static String USERNAME = "username" , PASSWORD1X = "password1x" ,
                        PASSWORD2X = "password2x" , EMAIL = "email" ,
                        FIRSTNAME = "firstname" , LASTNAME = "lastname" ;
  private User user_ ;
  private boolean isShowCancel_ = false ;
  public UIAccountForm() throws Exception {
    super("accountForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    setId("UIAccountForm") ;
    isShowCancel_ = false ;
    add(new UIStringInput(USERNAME, "").addValidator(NameValidator.class)) ;
    add(new UIStringInput(PASSWORD1X, "").
        setType(UIStringInput.PASSWORD).
        addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(PASSWORD2X, "").
        setType(UIStringInput.PASSWORD).
        addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(FIRSTNAME, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(LASTNAME, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(EMAIL, "").addValidator(EmailAddressValidator.class)) ;
    addValidator(ConfirmPasswordValidator.class) ;
  }
  
  public boolean isShowCancel() { return isShowCancel_ ;  }
  
  public void customizeNewAccountForm() {
    isShowCancel_ = false ;
    addActionListener(NewAccountListener.class, SAVE_ACTION) ;
  }
  public void customizeNewAccountFromUserList() {
    isShowCancel_ = true ;
    ActionListener[] actions = getActionListeners() ;
    boolean isExist = false ;
    for(int i = 0; i < actions.length; i ++) {
      if(actions[i].getClass().equals(CancelActionListener.class) ){
        isExist = true ;
      }        
    }
    if(!isExist){
      addActionListener(NewAccountListener.class, SAVE_ACTION) ;
      addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    }    
  }
  
  public void customizeUpdateAccountForm() {
    addActionListener(UpdateAccountListener.class, SAVE_ACTION) ;
    getUIStringInput(USERNAME).setEditable(false);
  }
  
  public void setEditingUser(String userName) throws Exception {
    OrganizationService service =
      (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
    user_ = service.getUserHandler().findUserByName(userName) ;
    setEditingUser(user_) ;
  }
  
  public void setEditingUser(User user) throws Exception {
    user_ = user;
    getUIStringInput(USERNAME).setValue(user_.getUserName());
    getUIStringInput(PASSWORD1X).setValue(user_.getPassword());
    getUIStringInput(PASSWORD2X).setValue(user_.getPassword());
    getUIStringInput(FIRSTNAME).setValue(user_.getFirstName());
    getUIStringInput(LASTNAME).setValue(user_.getLastName());
    getUIStringInput(EMAIL).setValue(user_.getEmail());
  }
  
  private String encodeSpecChar(String value){
    value = value.replaceAll("'", "&#39;");
    value = value.replaceAll("<", "&#60;");
    value = value.replaceAll(">", "&#62;");
    return value;
  }
  
  static public class ConfirmPasswordValidator implements Validator {
    public void validate(FacesContext context, UIComponent component,
                         Object value) throws ValidatorException {
      UIAccountForm uiForm = (UIAccountForm) component ;
      String p1 = uiForm.getUIStringInput(PASSWORD1X).getValue() ;
      String p2 = uiForm.getUIStringInput(PASSWORD2X).getValue() ;
      if(!p1.equals(p2)) {
        ResourceBundle res =  Util.getApplicationResourceBundle()  ;
        String message = res.getString("UIAccountForm.msg.password-error") ;
        throw new ValidatorException(new ExoFacesMessage(message)) ;
      }
    }
  }
  
  static public class NewAccountListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAccountForm uicomp = (UIAccountForm) event.getComponent() ;
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      String userName = uicomp.getUIStringInput(USERNAME).getValue() ;
      OrganizationService service =
        (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
      User newUser = service.getUserHandler().findUserByName(userName) ;
      String message = null ;
      if (newUser == null) {
        newUser = service.getUserHandler().createUserInstance() ;
        newUser.setUserName(userName) ;
        newUser.setPassword(uicomp.getUIStringInput(PASSWORD1X).getValue()) ;        
        newUser.setFirstName(uicomp.encodeSpecChar(
            uicomp.getUIStringInput(FIRSTNAME).getValue())) ;
        newUser.setLastName(uicomp.encodeSpecChar(
            uicomp.getUIStringInput(LASTNAME).getValue())) ;
        newUser.setEmail(uicomp.getUIStringInput(EMAIL).getValue()) ;
        service.getUserHandler().createUser(newUser, true);
        uicomp.reset() ;
        message = res.getString("UIAccountForm.msg.create-user-success") ;
      } else {
        message = res.getString("UIAccountForm.msg.user-already-exist") ;
      }
      iprovider.addMessage(new ExoFacesMessage(message)) ;
    }
  }
  
  static public class UpdateAccountListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAccountForm  uicomp = (UIAccountForm) event.getComponent() ;
      uicomp.user_.setPassword(uicomp.getUIStringInput(PASSWORD1X).getValue()) ;
      uicomp.user_.setFirstName(
          uicomp.encodeSpecChar(uicomp.getUIStringInput(FIRSTNAME).getValue())) ;
      uicomp.user_.setLastName(
          uicomp.encodeSpecChar(uicomp.getUIStringInput(LASTNAME).getValue())) ;
      uicomp.user_.setEmail(uicomp.getUIStringInput(EMAIL).getValue()) ;
      OrganizationService service =
        (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
      service.getUserHandler().saveUser(uicomp.user_, true) ;
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UIAccountForm.msg.update-user-success"))) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAccountForm  uicomp = (UIAccountForm) event.getComponent() ;
      uicomp.setRenderedSibling(UIListUser.class);
    }
  }
}