/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * this suite does: 
 *  1)Go to manager metadata on Admin the ECM tab 
 *  2)View some properties of dc:elementSet
 *  3)Apply some metadata
 *  4)Go to ECM file Explorer
 *  5)Go to Production 
 *  6)Go to Info Tab and view node properties
 *  7)Go to Metadata to save some properties
 *  8)Go to select workspace
 *  
 * Created by The eXo Platform SARL
 * Author : Hoa Nguyen
 *          hoa2501@yahoo.com
 * Apr 5, 2005
 */
public class AdminManagerMetadataSuite extends WebUnitSuite {
  static String DESC = "Go to Manager Admin page and test all basic actions" ;
  static String NAME = "AdminManagerMetadataSuite" ;
  String COMMENT = null;   
  public AdminManagerMetadataSuite() {    
    super(NAME, DESC) ;     
    goToPage("/Home/ecm") ;
    //Click Admin The ECM
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=addEditView"))      
    ) ;      
    //Click on Manager Metadata icon
    addWebUnit(
      new ClickURLLinkUnit("ClickManagerMetadata", "Click on Manage Metadata icon").
      setURL("uicomponent=UIECMAdminSummary&op=manageMetadata").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIMetadataManager&op=view&objectId=dc:elementSet"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickViewMetadata", "Click on view icon at row of dc:elementSet").
        setURL("uicomponent=UIMetadataManager&op=view&objectId=dc:elementSet").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIMetadataView&op=PhaseId[1].cancel")).
        addValidator(new ExpectTextValidator("Metadata view"))
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("GoToListMetadata", "Click on back icon to comeback list metadata").
        setURL("&uicomponent=UIMetadataView&op=PhaseId[1].cancel").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIMetadataManager&op=metadata"))
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("AddMetadata", "Click on metadata icon to add metadata").
        setURL("&uicomponent=UIMetadataManager&op=metadata").                  
        addValidator(new ExpectFormIdValidator("uiMappingMetadataForm"))
    ) ;    
    addWebUnit(
        new SubmitFormUnit("ApplySomeMetadata", "Click on apply some metadata").
        setFormId("uiMappingMetadataForm").
        setCheckBox("mix:referenceable",true).
        setCheckBox("mix:lockable",true).
        setField("op","save").       
        addValidator(new ExpectTextValidator("mix:referenceable")).         
        addValidator(new ExpectTextValidator("mix:lockable"))
    ) ;    
    addWebUnit(
        new ClickURLLinkUnit("GoToWorkspace", "Click on back icon to go to workspace").
        setURL("uicomponent=UIMetadataManager&op=back").                  
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=ECMPage&portal:action=changeTab&objectId=jcr"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnECMTab", "Click on ECM file Explorer").
        setURL("portal:componentId=ECMPage&portal:action=changeTab&objectId=jcr").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnProduction", "Click on Production icon").
        setURL("uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Info"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnInfoTab", "Click on Info Tab").
        setURL("&uicomponent=UIViewController&op=selectNode&objectId=Info").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewProperties"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickViewNodeProperties", "Click on View Node Properties icon").
        setURL("uicomponent=UIToolbar&op=viewProperties").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPropertiesManager&op=selectNode&objectId=UIMetadataFormController"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnMetadataTab", "Click on Metadata Tab").
        setURL("uicomponent=UIPropertiesManager&op=selectNode&objectId=UIMetadataFormController").                  
        addValidator(new ExpectFormIdValidator("uiMetadataForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveMetadata", "Click on save icon to save metadata").
        setFormId("uiMetadataForm").         
        setField("title","testMetadata").       
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPropertiesManager&op=selectNode&objectId=UIPropertiesInfo"))         
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPropertiesTab", "Click on Properties Tab").
        setURL("uicomponent=UIPropertiesManager&op=selectNode&objectId=UIPropertiesInfo").                  
        addValidator(new ExpectTextValidator("testMetadata")).
        addValidator(new ExpectTextValidator("dc:title")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=MainToolbar&op=selectWorkspace"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SelectWorkspace", "Click on Select workspace").
        setURL("uicomponent=MainToolbar&op=selectWorkspace").                  
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=ECMPage&portal:action=changeTab&objectId=ecm"))
    ) ;  
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}
