/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;

/**
 *  Add a new /test-folder directory
 *     1) Go to Production workspace
 *     2) Delete test-folder if does exist
 *     3) Add /test-folder directory
 *     4) Return to workspaces selection page
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserAddFolderSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page and add a new folder" ;
  static String NAME = "UserAddFolderSuite" ;
  
  public UserAddFolderSuite() {    
    super(NAME, DESC) ;        
    goToPage("/Site Map/ecm") ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
        setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectWorkspace"))      
    ) ; 
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;  
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").         		  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ; 
    //-----Goto /home/users/#{client.name}
   addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}")
    ) ;
    //-----Delete test-folder if does exist
    
    addWebUnit(
      new ClickURLLinkUnit("DeleteExistedFolder", "Delete if test-folder is existed").
      setURL("&uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder").
      addCondition(new HaveLinkWithTextCondition("test-folder")).
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder has been removed."))
    ) ;        
    
    //-----Add folder directory
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddFolder", "click on Add Folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").                  
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;    
    
    //----------Add nt:unstructured folder
    addWebUnit(
      new SubmitFormUnit("Add Unstructured Folder", "Select type is nt:unstructured and click on Save button").
      setFormId("folderForm").   
      setField("type","nt:unstructured").
      setField("name","test-folder").
      setField("op","save").
      addValidator(new ExpectTextValidator("test-folder"))      
    ) ;
    
    //-----Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder"))      
    ) ; 
    
    //-----Go to test-folder
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder", "click on test-folder").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder")      
    ) ;        
    
  }                 
  public java.util.List getRequiredSuites() {    
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}