/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 1) once a node has been versionned, go to its parent node and ensure the "check in" icon appear
 2) Check in and check out several time, modyfing the text by editing it each time
 3) Go back to the node and view version, you should see a tree of versions
 4) Compare text diff version
 5) Test add and remove labels for versions 
 6) Restore an older version
 7) Go back to parent node and check in/out some more time
 8) Verify the branch has been created
 9) Test merge of 2 branches (need UI from HUNG see just after)

 HUNG will have to fix the css and HTML there (using groovy instead of velocity for nested macros)
 HUNG will have to also add icons for jcr:rootVersion
 HUNG will have to add UI to allow version merge
 
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
*/
public class UserManageVersionsSuite extends WebUnitSuite {  
  static String DESC = "Go to Ecm Portlet User page, click on Manage Versions button and test all basic actions" ;
  static String NAME = "UserManageVersionsSuite" ;
  String COMMENT  = "You must run AdminViewSuite before run this suite";
  
  public UserManageVersionsSuite() {    
    super(NAME, DESC) ;
    addWebSuite(new UserAddFolderSuite()); 
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageVersions"))      
    ) ;
    
		//-----Manage Versions (test-folder's version)
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageVersions", "click on Manage Versions button").
      setURL("&uicomponent=UIToolbar&op=manageVersions").           
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIVersionInfo&op=enableVersion"))      
    ) ;         
    
    //----------Enable Versioning
    addWebUnit(
      new ClickURLLinkUnit("ClickOnEnableVersioning", "select Enable Versioning link").
      setURL("&uicomponent=UIVersionInfo&op=enableVersion").                  
      addValidator(new ExpectTextValidator("You need to save the node to be able to access the version tree."))      
    ) ;
    
    //----------Saved Session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    
    //----------Once a node has been versionned, go to its parent node and ensure the "check in" icon appear    
    addWebUnit(
      new ClickURLLinkUnit("BackToParentNode", "Click on privious node").
      setURL("&uicomponent=MainToolbar&op=back").             
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=checkin&objectId=/home/users/#{client.name}/test-folder"))      
    ) ;   
    
    //----------Check in and check out several time, modyfing the text by editing it each time
    for ( i in [1,2,3]){    
      addWebUnit(
        new ClickURLLinkUnit("CheckIn", "Click on checkin button").
        setURL("uicomponent=UIDocumentInfo&op=checkin&objectId=/home/users/#{client.name}/test-folder").        
        addValidator(new ExpectTextValidator((String)i)).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=checkout&objectId=/home/users/#{client.name}/test-folder"))        
      ) ;
      addWebUnit(
        new ClickURLLinkUnit("CheckOut", "Click on checkout button").
        setURL("uicomponent=UIDocumentInfo&op=checkout&objectId=/home/users/#{client.name}/test-folder").
        addValidator(new ExpectTextValidator((String)(i))).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=checkin&objectId=/home/users/#{client.name}/test-folder"))      
      ) ;
    }
   
   //Go back to the node and view version, you should see a tree of versions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnTest-folder", "Click on test-folder").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder").
      addValidator(new ExpectTextValidator("test-folder")) //This node does not have any child
    ) ;
    
    addWebUnit(
      new ClickURLLinkUnit("ViewNodeVersion", "Click manage versions").
      setURL("&uicomponent=UIToolbar&op=manageVersions").
      addValidator(new ExpectTextValidator("1")).
      addValidator(new ExpectTextValidator("2")).
      addValidator(new ExpectTextValidator("3")).
      addValidator(new ExpectTextValidator("(Base Version)")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIVersionInfo&op=removeLabel"))
    ) ;
    
    COMMENT = "Because node version's URL is random so you can not yet test for Add Labels";
    addWebUnit(
      new TODO("Can'tAddLabel", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    COMMENT = "Because node version's URL is random so you can not yet test for Restore an older version' ";
    addWebUnit(
      new TODO("Can'tRestoreOldVersion", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    
    //----------Click on add label     
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddLabelOK", "Click on add label").
        setURL("uicomponent=UIVersionInfo&op=addLabel&objectId=/jcr:system/jcr:versionStorage").
        addValidator(new ExpectTextValidator("Add Version Label"))
    ) ;
    
    //---------------Click on Save to add label
  
    addWebUnit(
        new SubmitFormUnit("ClickOnAddLabel", "Click on Add Label  is empty").
        setFormId("UILabelForm").
        setField("name","test add label").                  
        setField("op","save").
        addValidator(new ExpectTextValidator("test add label"))
    ) ; 
    
    //----------Click on Remove version
    COMMENT = "Because node version's URL is random so you can not yet test for remove an version' ";
    addWebUnit(
      new TODO("Can'tRemoveVersion", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    
    //----------Click on View version
    COMMENT = "Because node version's URL is random so you can not yet test for view an version' ";
    addWebUnit(
      new TODO("Can'tViewVersion", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActionsTab", "click on Actions tab").
      setURL("uicomponent=UIViewController&op=selectNode&objectId=Actions").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    
    //----- Add a new document 
    addWebUnit(
      new ClickURLLinkUnit("AddNewDocument ", "click on add document icon").
      setURL("uicomponent=UIToolbar&op=addDocument").           
      addValidator(new ExpectFormIdValidator("selectDocumentForm"))      
    ) ;  
    
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:senMailAction").
        setFormId("selectDocumentForm").   
        setField("documentType","nt:file").
        setField("op","changeDocument").
        addValidator(new ExpectTextValidator("documentForm"))
      ) ;
   
   addWebUnit(
        new SubmitFormUnit("SubmitAddDocForm", "Click on save a new document ").
        setFormId("documentForm").    
        setField("docName","VersionLast").
        setField("summary","VersionLast").
        setField("op","save").
        addValidator(new ExpectTextValidator("VersionLast"))  
    ) ; 
    
    //-----Saved Session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    
    //Go back to parent node and check in/out some more time
    addWebUnit(
      new ClickURLLinkUnit("BackToParentNode", "Click on privious node").
      setURL("portal:componentId=jcr&portal:type=action&portal:isSecure=false&uicomponent=MainToolbar&op=back").             
      addValidator(new ExpectTextValidator("3"))      
    ) ;
    
    for ( i in [4,5,6]){    
      addWebUnit(
          new ClickURLLinkUnit("CheckIn", "Click on checkin button").
          setURL("uicomponent=UIDocumentInfo&op=checkin&objectId=/home/users/#{client.name}/test-folder").            
          addValidator(new ExpectTextValidator((String)i)).
          addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=checkout&objectId=/home/users/#{client.name}/test-folder"))
        ) ;
      addWebUnit(
          new ClickURLLinkUnit("CheckOut", "Click on checkout button").
          setURL("uicomponent=UIDocumentInfo&op=checkout&objectId=/home/users/#{client.name}/test-folder").
          addValidator(new ExpectTextValidator((String)(i))).
          addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=checkin&objectId=/home/users/#{client.name}/test-folder"))      
        ) ;
    }
    
    //Verify the branch has been created    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnTest-folder", "Click on test-folder").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder").
      addValidator(new ExpectTextValidator("test-folder"))
    ) ;
    
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin")                
    ) ;
    
    addWebUnit(
      new ClickURLLinkUnit("ViewNodeVersion", "Click manage versions").
      setURL("&uicomponent=UIToolbar&op=manageVersions").
      addValidator(new ExpectTextValidator("4")).
      addValidator(new ExpectTextValidator("5")).
      addValidator(new ExpectTextValidator("6")).
      addValidator(new ExpectTextValidator("(Base Version)")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIVersionInfo&op=removeLabel"))
    ) ;
    
    //----------Click on restore version
    addWebUnit(
        new ClickURLLinkUnit("RestoreVersion", "Click on restore version icon").
        setURL("uicomponent=UIVersionInfo&op=restoreVersion&objectId=/jcr:system/jcr:versionStorage/").
        addValidator(new ExpectTextValidator("The version 1 was restored successfully."))
    ) ;
    
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActionsTab", "click on Actions tab").
      setURL("uicomponent=UIViewController&op=selectNode&objectId=Actions")      
    ) ;
    
    //Go back to parent node and check out
    addWebUnit(
      new ClickURLLinkUnit("BackToParentNode", "Click on privious node").
      setURL("portal:componentId=jcr&portal:type=action&portal:isSecure=false&uicomponent=MainToolbar&op=back").             
      addValidator(new ExpectTextValidator("1"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("CheckOut", "Click on checkout button").
        setURL("uicomponent=UIDocumentInfo&op=checkout&objectId=/home/users/#{client.name}/test-folder").
        addValidator(new ExpectTextValidator("1")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=checkin&objectId=/home/users/#{client.name}/test-folder"))      
    ) ;
    
    COMMENT = "You can not yet test merge version, because you have not UI for merge version";   
    addWebUnit(
      new TODO("Can'tMergeVersion", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    
    //-----Delete test-folder 
    addWebUnit(
      new ClickURLLinkUnit("DeleteTestFolder", "Delete  test-folder ").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder").
      addCondition(new HaveLinkWithURLCondition("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder")).
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder has been removed."))
    ) ;  
    
    //-----Saved Session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    
    //Select Workspace
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}