/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * This suite does:
 *    1) Go to productions
 *    2) Go to current user and add 3 folder : test-folder1,test-folder2,test-folder3
 *    3) Click on Preferences icon and select 'Show ClipBoard'
 *    4) Click on copy icon of test-folder1
 *    5) Click on copy icon of test-folder2 
 *    6) Click on cut icon of test-folder2
 *    7) Click on copy icon of test-folder3
 *    8) Go to test-folder3 and parse test-folder1 and test-folder2
 *    9) Delete all actions
 *   10) Go to publication
 *   11) Click on prefenreces icon and test some actions                     
 *    
 * Created by The eXo Platform SARL
 * Author : Hoa Nguyen Thi
 *          hoa.nguyen@exoplatform.com, hoa2501@yahoo.com
 * Apr 19, 2006
 */
public class UserPreferencesSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page and test some basic ations for Preference" ;
  static String NAME = "UserPreferencesSuite" ;
  public UserPreferencesSuite() {    
    super(NAME, DESC) ;  
    goToPage("/Site Map/ecm") ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
        setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectWorkspace"))      
    ) ; 
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;  
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ; 
    //-----Goto /home/users/#{client.name}
   addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}")
    ) ;
    //Create a /test-folder2 directory    
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTestfolder1", "click delete button if test-folder1 existed").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder1").      
      addValidator(new ExpectTextValidator("The node /test-folder1 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder1"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTestfolder1", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder1").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder1"))                             
    ) ;
    //Create /test-folder2
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTest-folder2", "click delete button if test-folder2 existed").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder2").      
      addValidator(new ExpectTextValidator("The node /test-folder2 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder2"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder2", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder2").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder2"))      
    ) ;
    //Create /test-folder4
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTest-folder3", "click delete button if test-folder3 existed").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder3").      
      addValidator(new ExpectTextValidator("The node /test-folder3 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder3"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder3", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder3").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder3"))      
    ) ;
    //-----Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;   
    //-----Refresh session
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on Refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").  
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPreference", "Click on Preference icon").
        setURL("&uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
    ) ;
    addWebUnit(
        new SubmitFormUnit("SetUpPreference", "Set up your browsing preferences is Show ClipBoard").
        setFormId("preferencesForm").  
        setCheckBox("showClipBoard",true).
        setField("op","save")        
    ) ;
    //----------click on copy icon of test-folder1
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCopyIcon", "Click on copy icon of test-folder1").
        setURL("uicomponent=UIDocumentInfo&op=copy&objectId=/home/users/#{client.name}/test-folder1").      
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder1 has been copied."))      
    ) ;
    //----------Click on copy icon of test-folder2
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCopyIcon", "Click on copy icon of test-folder2").
        setURL("uicomponent=UIDocumentInfo&op=copy&objectId=/home/users/#{client.name}/test-folder2").                  
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder2 has been copied."))      
    ) ;
    //----------Click on cut icon of test-folder2
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCutIcon", "Click on cut icon of test-folder2").
        setURL("uicomponent=UIDocumentInfo&op=cut&objectId=/home/users/#{client.name}/test-folder2").                  
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder2 has been cut."))      
    ) ;
    //----------Click on copy icon of test-folder3
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCopyIcon", "Click on Copy icon of test-folder3 ").
        setURL("uicomponent=UIDocumentInfo&op=copy&objectId=/home/users/#{client.name}/test-folder3").                  
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder3 has been copied."))      
    ) ;
    //-----Go to test-folder3
    addWebUnit(
        new ClickURLLinkUnit("SelectTestFolder3", "Click on test-folder3 ").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder3").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIJCRExplorer&op=pasteNode&objectId=2"))      
    ) ;
    //----------parse test-folder1
    addWebUnit(
        new ClickURLLinkUnit("ParseWithActionCopy", "Click on copy icon on Preference").
        setURL("uicomponent=UIJCRExplorer&op=pasteNode&objectId=1").                  
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder1 has been pasted at the location /home/users/#{client.name}/test-folder3/test-folder1. This operation has already been save and you do not need to press the floppy icon."))      
    ) ;
    //----------parse test-folder2
    addWebUnit(
        new ClickURLLinkUnit("ParseWithActionCut", "Click on cut icon on Preference").
        setURL("uicomponent=UIJCRExplorer&op=pasteNode&objectId=2").                  
        addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder2 has been pasted at the location /home/users/#{client.name}/test-folder3/test-folder2. This operation has already been save and you do not need to press the floppy icon."))      
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;   
    //-----Refresh session
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on Refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").  
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ;
    //----------Delete action copy 
    addWebUnit(
        new ClickURLLinkUnit("DeleteActionCopy", "Click on delete icon on Preference").
        setURL("&uicomponent=UIJCRExplorer&op=clear&objectId=1").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIJCRExplorer&op=pasteNode&objectId=3",0))      
    ) ;
    //----------Delete all action
    addWebUnit(
        new ClickURLLinkUnit("DeleteAllAction", "Click on delete icon on Preference").
        setURL("&uicomponent=UIJCRExplorer&op=clearAll").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIJCRExplorer&op=pasteNode&objectId=",0))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("HideClipboard", "Click on hideClipboard icon").
        setURL("uicomponent=UIJCRExplorer&op=hideClipboard").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIJCRExplorer&op=hideClipboard",0))      
    ) ;
    //Return to workspace selection
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;    
    //Go to productions/cms/publication
    addWebUnit(
        new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
        setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectWorkspace"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCms", "click on cms link").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/backup",0)).      
        addValidator(new ExpectLinkWithURLValidator("uicomponent=MainToolbar&op=preferences"))
    ) ;
    //-----Click on preferences icon
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("Don'tShowNonDocType", "Don't select checkbox 'Show non document type'").
        setFormId("preferencesForm").
        setCheckBox("showNonDocumentType",true).
        setField("op","save").
        addValidator(new ExpectTextValidator("backup"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("ShowNonDocType", "Select checkbox 'Show non document type'").
        setFormId("preferencesForm").
        setCheckBox("showNonDocumentType",false).
        setField("op","save").
        addValidator(new ExpectNoTextValidator("backup"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("GoToWorkSpace", "Go to workspace").
        setURL("uicomponent=MainToolbar&op=selectWorkspace").                  
        addValidator(new ExpectTextValidator("workspace"))      
    ) ; 
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}