/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;


/**
 * This suite does :
 *    1)Go to current user
 *    2)Add new folder has name is test-folder
 *    3)Add new document with type is exo:article which has name is test-doc
 *    4)Add test-doc to News category on cms
 *    5)Login with account exoadmin and add relationDoc document with type is exo:article  to cms/publication
 *    6)Check some status view relation
 *    7)Go back current user and add relation is relationDoc for test-doc
 *    8)Login with account exoadmin and go to browse category to check exist status of test-doc and relationDoc on News category
 *    9)Go back current user to delete relation,test-doc,test-folder
 *   10)Login with account exoadmin and delete relationDoc
 *   11)Go back current user
 *   
 * Created by The eXo Platform SARL
 * Author :Hoa Nguyen Thi 
 *         hoa2501@yahoo.com
 * Apr 12, 2006
 */
public class UserRelationSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on test-folder and to do somethings" ;
  static String NAME = "UserRelationSuite" ;
  public UserRelationSuite() {    
    super(NAME, DESC) ; 
    
    //-----Login with admin account
    /*
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    ); 
    */
    addWebUnit(
        new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
        setURL("&uicomponent=UILogout&op=logout").        
        addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ; 
        
    String[] roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ; 
    
    goToPage("/Site Map/ecm") ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
        setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectWorkspace"))      
    ) ; 
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;
    
    // Goto publications folder and create relationDoc with nodetype exo:article
		addWebUnit(
        new ClickURLLinkUnit("ClickOnCms", "click on home cms").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms").             
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications"))           
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
        setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))
    ) ;
    
    //----------Add new Doc has type is exo:article
    addWebUnit(
        new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
        setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("AddNewDocument", "Click on add document icon").
        setURL("uicomponent=UIToolbar&op=addDocument").               
        addValidator(new ExpectFormIdValidator("documentForm"))      
    ) ; 
    
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
        setFormId("selectDocumentForm").   
        setField("documentType","exo:article").
        setField("op","changeDocument").
        addValidator(new ExpectTextValidator("Article"))
    ) ;
    
    addWebUnit(
        new SubmitFormUnit("SaveNewDocument", "click on save button").
        setFormId("documentForm").   
        setField("docName","relationDoc").
        setField("title","relationDoc").
        setField("op","save")
        
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("Saved Session after create relationDoc", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("relationDoc"))      
    ) ;    
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
        setURL("&uicomponent=UILogout&op=logout").        
        addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ; 
    
    //-----Back to current user
    String[] userRoles = ["user"] ;
    addWebSuite(new UserLoginSuite("#{client.name}", "#{client.name}", userRoles)) ;
    
    /*
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );
     
    */
    //Add new folder
    addWebSuite(new UserAddFolderSuite());
    
    // Create acrticle Document
     addWebUnit(
        new ClickURLLinkUnit("AddNewDocument", "Click on add document icon").
        setURL("uicomponent=UIToolbar&op=addDocument").               
        addValidator(new ExpectFormIdValidator("documentForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
        setFormId("selectDocumentForm").   
        setField("documentType","exo:article").
        setField("op","changeDocument").
        addValidator(new ExpectTextValidator("Article"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveNewDocument", "click on save button").
        setFormId("documentForm").   
        setField("docName","test-doc").
        setField("title","test-doc").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-doc"))
    ) ;
    
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-doc"))      
    ) ;
    
    // Goto test-doc and add relation
    addWebUnit(
         new ClickURLLinkUnit("SelectTest-doc ", "Click on test-doc link").
         setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-doc").        
         addValidator(new ExpectTextValidator("test-doc"))      
     ) ;
     
    addWebUnit(
        new ClickURLLinkUnit("SelectAdminTab", "Select Admin tab").
        setURL("uicomponent=UIViewController&op=selectNode&objectId=Admin").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=manageRelations"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("SelectManageRelations", "Select Manage Relation").
        setURL("uicomponent=UIToolbar&op=manageRelations").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIRelationsList&op=add"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddRelation", "Click on add relation icon").
        setURL("uicomponent=UIRelationsList&op=add").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms/publications"))      
    ) ;   
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "Click on add relation icon").
        setURL("&uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms/publications").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrRelationBrowser&op=selectNode&objectId=/cms/publications/relationDoc"))      
    ) ; 
     
    addWebUnit(
        new ClickURLLinkUnit("SelectRelationsDoc", "Select relationDoc").
        setURL("uicomponent=JcrRelationBrowser&op=selectNode&objectId=/cms/publications/relationDoc").        
        addValidator(new ExpectTextValidator("/cms/publications/relationDoc"))      
    ) ;
    
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession")    
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnBack", "Click on back").
        setURL("uicomponent=UIRelationsList&op=PhaseId[1].cancel").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc"))      
    ) ; 
    
    addWebUnit(
        new ClickURLLinkUnit("View RelationDoc", "Click on relationDoc").
        setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc").        
        addValidator(new ExpectTextValidator("relationDoc"))      
    ) ;
    
   addWebUnit(
      new ClickURLLinkUnit("Back to test folder", "Click on Back To test folder").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectTextValidator("relationDoc")).
      addValidator(new ExpectTextValidator("Article"))
    ) ; 
    
    addWebUnit(
         new ClickURLLinkUnit("SelectInfoTab ", "Select Info tab").
         setURL("uicomponent=UIViewController&op=selectNode&objectId=Info").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
     ) ;
     
     addWebUnit(
         new ClickURLLinkUnit("ClickOnViewRelation ", "Click on view relation").
         setURL("uicomponent=UIToolbar&op=viewRelations").        
         addValidator(new ExpectNoTextValidator("This node hasn't been relationable yet")).      
         addValidator(new ExpectTextValidator("Related Document"))
     ) ;
     
     addWebUnit(
         new ClickURLLinkUnit("ClickOnrelationDoc ", "Click on relationDoc").
         setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc").                                  
         addValidator(new ExpectTextValidator("relationDoc"))      
     ) ;
   
    //Return to workspace selection
     addWebUnit(
       new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
       setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
       addValidator(new ExpectTextValidator("Workspaces"))      
     ) ;    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}