/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.user;

import condition.* ;
import suites.share.*;
import suites.share.UserLoginSuite;
import suites.share.NewAccountSuite;
import suites.share.ViewUserProfileSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: OrganizationPortletSuite.java,v 1.3 2004/10/22 14:23:55 tuan08 Exp $
 **/
public class UsersManagementSuite extends WebUnitSuite {
  static String DESC = "Go to Users Management page and test all the basic actions at the users tab" ;
  static String NAME = "UsersManagementSuite" ;
  final static public String EXOPLATFORM_ACCOUNT = "exoplatform" ;
    
  public UsersManagementSuite() {    
    super(NAME, DESC) ;
    init("/Site Map/Management", "/Site Map/Add User") ;
  }  
  public UsersManagementSuite(String path, String reg_path) {
    super(NAME, DESC) ;
    init(path, reg_path) ;
  }  
  public void  init(path, reg_path) {    
    goToPage(path) ; 
    addWebUnit(
      new ClickURLLinkUnit("GoToUsersTab", "Click on the users tab").
      setURL("op=selectNode&objectId=usernode").
      addValidator(new ExpectTextValidator("Email"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitSearcUserForm", "search for exoplatform user").
      setFormId("quickSearchForm").
      setField("term", EXOPLATFORM_ACCOUNT).
      setField("op","search")
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedAccount", "Delete the test existed account").
      setURL("op=delete&objectId="+EXOPLATFORM_ACCOUNT).
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+EXOPLATFORM_ACCOUNT))
    ) ;
    goToPage(reg_path) ;
    //Create exoplatform account , the account may already existed , it is ok
    addWebSuite(new NewAccountSuite(EXOPLATFORM_ACCOUNT, EXOPLATFORM_ACCOUNT)) ;
    goToPage(path);
    //Search exoplatform account
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for exoplatform user").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term",EXOPLATFORM_ACCOUNT).
      setField("op","search")
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserProfile", "Click the view user profile icon").
      setURL("op=viewProfile&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectTextValidator("Info"))
    ) ; 
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("back")
    ) ;
    //View exoplatform account info
    addWebUnit(
      new ClickURLLinkUnit("ClickViewUserInfo", "Click the view user info icon").
      setURL("op=viewUserInfo&objectId="+ EXOPLATFORM_ACCOUNT).
      addValidator(new ExpectFormIdValidator("membershipForm"))
    ) ;
    addWebUnit(
      new ClickLinkWithText("ClickBackLink", "Go back to the user list screen").
      setTextLink("back")
    ) ;
    //Add new user
    addWebUnit(
        new ClickURLLinkUnit("ClickAddUserIcon", "Click the add user icon").
        setURL("&uicomponent=UIListUser&op=addUser").
        addValidator(new ExpectFormIdValidator("accountForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitAddUserForm", "Click on save icon").
        setFormId("accountForm").
        setField("username","user1").
        setField("password1x","user").
        setField("password2x","user").
        setField("firstname","user's").
        setField("lastname","user's").
        setField("email","user1@exoplatform.com").
        setField("op","save").
        addValidator(new ExpectTextValidator("New user has been created"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitAddUserForm", "Click on save icon").
        setFormId("accountForm").
        setField("username","user2").
        setField("password1x","user").
        setField("password2x","user").
        setField("firstname","user<").
        setField("lastname","user<").
        setField("email","user2@exoplatform.com").
        setField("op","save").
        addValidator(new ExpectTextValidator("New user has been created"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitAddUserForm", "Click on save icon").
        setFormId("accountForm").
        setField("username","user3").
        setField("password1x","user").
        setField("password2x","user").
        setField("firstname","user>").
        setField("lastname","user>").
        setField("email","user3@exoplatform.com").
        setField("op","save").
        addValidator(new ExpectTextValidator("New user has been created"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnCancelIcon", "Go back to list user").
        setFormId("accountForm").
        setField("op","PhaseId[1].cancel").
        addValidator(new ExpectNoTextValidator("user's"))
    ) ;
    //Search user account with firstname and lastname have sign "'"
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for 'user's' user").
      setFormId("quickSearchForm").
      setField("term","*'*").
      setField("searchField","firstname").      
      setField("op","search").
      addValidator(new ExpectTextValidator("user1@exoplatform.com"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitSearchUserForm", "search for 'user's' user").
        setFormId("quickSearchForm").
        setField("term","*'*").
        setField("searchField","lastname").      
        setField("op","search").
        addValidator(new ExpectTextValidator("user1@exoplatform.com"))
    ) ;
    //Delete user account
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteAccount", "Delete Account").
      setURL("op=delete&objectId=user1").
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId=user1"))
    ) ;
    //Search user account with firstname and lastname have sign "<"
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for 'user's' user").
      setFormId("quickSearchForm").
      setField("term","*<*").
      setField("searchField","firstname").      
      setField("op","search").
      addValidator(new ExpectTextValidator("user2@exoplatform.com"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitSearchUserForm", "search for 'user's' user").
        setFormId("quickSearchForm").
        setField("term","*<*").
        setField("searchField","lastname").      
        setField("op","search").
        addValidator(new ExpectTextValidator("user2@exoplatform.com"))
    ) ;
    //Delete user account
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteAccount", "Delete Account").
      setURL("op=delete&objectId=user2").
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId=user2"))
    ) ;
    //Search user account with firstname and lastname have sign "'"
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for 'user's' user").
      setFormId("quickSearchForm").
      setField("term","*>*").
      setField("searchField","firstname").      
      setField("op","search").
      addValidator(new ExpectTextValidator("user3@exoplatform.com"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitSearchUserForm", "search for 'user's' user").
        setFormId("quickSearchForm").
        setField("term","*>*").
        setField("searchField","lastname").      
        setField("op","search").
        addValidator(new ExpectTextValidator("user3@exoplatform.com"))
    ) ;
    //Delete user account
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteAccount", "Delete Account").
      setURL("op=delete&objectId=user3").
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId=user3"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitSearchUserForm", "search for user to delete").
        setFormId("quickSearchForm").
        setField("searchField","username").
        setField("term",((String)EXOPLATFORM_ACCOUNT).substring(0,((String)EXOPLATFORM_ACCOUNT).length()-1)+"*" ).
        setField("op","search").
        addValidator(new ExpectTextValidator(EXOPLATFORM_ACCOUNT))
    ) ;
    //Delete exoplatform account
    addWebUnit(
      new ClickURLLinkUnit("ClickDeleteAccount", "Delete Account").
      setURL("op=delete&objectId="+EXOPLATFORM_ACCOUNT).
      addCondition(new HaveLinkWithURLCondition("op=delete&objectId="+EXOPLATFORM_ACCOUNT))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SubmitSearchUserForm", "search for user to delete").
      setFormId("quickSearchForm").
      setField("searchField","username").
      setField("term",((String)EXOPLATFORM_ACCOUNT).substring(0,((String)EXOPLATFORM_ACCOUNT).length()-1)+"*" ).
      setField("op","search").
      addValidator(new ExpectNoTextValidator(EXOPLATFORM_ACCOUNT))
    ) ;
  }
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }
}
